@@ -1,10 +1,10 @@
 /*
 
-  $Id: cfgreader.c,v 1.47 2004/02/27 19:23:20 bozo Exp $
+  $Id: cfgreader.c,v 1.58 2005/04/20 22:33:36 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,13 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999      Tim Potter
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2000      Jan Derfinak
+  Copyright (C) 2001      Jan Kratochvil
+  Copyright (C) 2001-2005 Pawel Kot
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2005      Bastien Nocera
 
   Config file (/etc/gnokiirc and ~/.gnokiirc) reader.
 
@@ -30,6 +36,9 @@
 
 */
 
+
+/* for strndup */
+#define _GNU_SOURCE
 #include "config.h"
 #include "compat.h"
 #include "misc.h"
@@ -46,36 +55,25 @@
 API struct gn_cfg_header *gn_cfg_info;
 static gn_config gn_config_default, gn_config_global;
 
-/* Read configuration information from a ".INI" style file */
-struct gn_cfg_header *cfg_file_read(const char *filename)
+struct gn_cfg_header *cfg_memory_read(const char **lines)
 {
-	FILE *handle;
-	char *line;
-	char *buf;
+	char *line, *buf;
 	struct gn_cfg_header *cfg_info = NULL, *cfg_head = NULL;
+	int i;
 
-	/* Error check */
-	if (filename == NULL) {
+	/* Check whether the given memory location is not null */
+	if (lines == NULL) {
+		dprintf("cfg_memory_read - passed nil data\n");
 		return NULL;
+	} else {
+		dprintf("Opened configuration file from memory\n");
 	}
 
-	/* Initialisation */
-	if ((buf = (char *)malloc(255)) == NULL) {
-		return NULL;
-	}
-
-	/* Open file */
-	if ((handle = fopen(filename, "r")) == NULL) {
-		dprintf("cfg_file_read - open %s: %s\n", filename, strerror(errno));
-		return NULL;
-	}
-	else
-		dprintf("Opened configuration file %s\n", filename);
-
 	/* Iterate over lines in the file */
-	while (fgets(buf, 255, handle) != NULL) {
+	for (i = 0; lines[i] != NULL; i++) {
 
-		line = buf;
+		line = strdup (lines[i]);
+		buf = line;
 
 		/* Strip leading, trailing whitespace */
 		while(isspace((int) *line))
@@ -94,7 +92,6 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 
 			/* Allocate new heading entry */
 			if ((heading = (struct gn_cfg_header *)malloc(sizeof(*heading))) == NULL) {
-				free(buf);
 				return NULL;
 			}
 
@@ -121,19 +118,19 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 
 			dprintf("Added new section %s\n", heading->section);
 
+			free(buf);
+
 			/* Go on to next line */
 			continue;
 		}
 
 		/* Process key/value line */
-
 		if ((strchr(line, '=') != NULL) && cfg_info != NULL) {
 			struct gn_cfg_entry *entry;
 			char *value;
 
 			/* Allocate new entry */
 			if ((entry = (struct gn_cfg_entry *)malloc(sizeof(*entry))) == NULL) {
-				free(buf);
 				return NULL;
 			}
 
@@ -141,10 +138,10 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 			memset(entry, '\0', sizeof(*entry));
 
 			value = strchr(line, '=');
-			*value = '\0';		/* Split string */
+			*value = '\0';	/* Split string */
 			value++;
 
-			while(isspace((int) *value)) {      /* Remove leading white */
+			while (isspace((int) *value)) { /* Remove leading white */
 				value++;
 			}
 
@@ -169,19 +166,117 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 
 			dprintf("Adding key/value %s/%s\n", entry->key, entry->value);
 
+			free(buf);
+
 			/* Go on to next line */
 			continue;
 		}
 
 			/* Line not part of any heading */
 		fprintf(stderr, "Orphaned line: %s\n", line);
+
+		free(buf);
 	}
 
-	free(buf);
 	/* Return pointer to configuration information */
 	return cfg_head;
 }
 
+#define READ_CHUNK_SIZE 64
+
+/* Read configuration information from a ".INI" style file */
+struct gn_cfg_header *cfg_file_read(const char *filename)
+{
+	FILE *handle;
+	char *lines, *line_begin, *line_end, *pos;
+	char **split_lines;
+	int read, ret, num_lines, i, copied;
+	struct gn_cfg_header *header = NULL;
+
+	/* Open file */
+	if ((handle = fopen(filename, "r")) == NULL) {
+		dprintf("cfg_file_read - open %s: %s\n", filename, strerror(errno));
+		goto out;
+	} else {
+		dprintf("Opened configuration file %s\n", filename);
+	}
+
+	/* Read the lines */
+	lines = NULL;
+	read = 0;
+	do {
+		lines = realloc(lines, read + READ_CHUNK_SIZE);
+		if (!lines)
+			goto err_read;
+
+		ret = fread(lines + read, 1, READ_CHUNK_SIZE, handle);
+		/* Read error */
+		if (ret < 0 && feof(handle) == 0)
+			goto err_read;
+
+		/* Overflow */
+		if (read + ret < read)
+			goto err_read;
+
+		read += ret;
+	} while (ret > 0);
+
+	fclose(handle);
+	lines = realloc(lines, read + 1);
+	lines[read] = '\0';
+
+	/* Now split the lines */
+	split_lines = NULL;
+	line_begin = lines;
+	num_lines = 0;
+	copied = 0;
+	while ((pos = strchr(line_begin, '\n')) && copied < read) {
+		char *buf;
+
+		if (!pos) {
+			line_end = lines + read;
+		} else {
+			line_end = pos;
+		}
+
+		num_lines++;
+		buf = (char *)strndup(line_begin, line_end - line_begin);
+		split_lines = realloc(split_lines,
+				(num_lines + 1) * sizeof(char*));
+		split_lines[num_lines - 1] = buf;
+
+		if (pos) {
+			copied += (line_end + 1 - line_begin);
+			line_begin = line_end + 1;
+		}
+	}
+
+
+	free(lines);
+	if (split_lines == NULL)
+		goto out;
+	split_lines[num_lines] = NULL;
+
+	/* Finally, load the configuration from the split lines */
+	header = cfg_memory_read((const char **)split_lines);
+
+	/* Free the split_lines */
+	for (i = 0; split_lines[i] != NULL; i++) {
+		free(split_lines[i]);
+	}
+	free(split_lines);
+
+	goto out;
+
+err_read:
+	fclose(handle);
+	if (lines)
+		free(lines);
+
+out:
+	return header;
+}
+
 /*  Write configuration information to a config file */
 int cfg_file_write(struct gn_cfg_header *cfg, const char *filename)
 {
@@ -306,6 +401,8 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 			cfg->connection_type = GN_CT_Irda;
 		else if (!strcasecmp(val, "bluetooth"))
 			cfg->connection_type = GN_CT_Bluetooth;
+		else if (!strcasecmp(val, "dku2"))
+			cfg->connection_type = GN_CT_DKU2;
 #ifndef WIN32
 		else if (!strcasecmp(val, "tcp"))
 			cfg->connection_type = GN_CT_TCP;
@@ -388,6 +485,13 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 		return false;
 	}
 
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "sm_retry")))
+		cfg->sm_retry = def->sm_retry;
+	else if (sscanf(val, " %d %c", &cfg->sm_retry, &ch) != 1) {
+		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "sm_retry", val);
+		return false;
+	}
+
 	return true;
 }
 
@@ -410,53 +514,83 @@ static bool cfg_get_log_target(gn_log_target *t, const char *opt)
 	return true;
 }
 
-API int gn_cfg_read(char **bindir)
+#define MAX_PATH_LEN 200
+API int gn_cfg_read_default()
 {
 	char *homedir;
-	char rcfile[200];
-	char *default_bindir     = "/usr/local/sbin/";
+	char rcfile[MAX_PATH_LEN];
+#ifdef WIN32
+	const char *globalrc = "gnokiirc";
+
+	homedir = getenv("HOMEDRIVE");
+	strncpy(rcfile, homedir ? homedir : "", MAX_PATH_LEN);
+	homedir = getenv("HOMEPATH");
+	strncat(rcfile, homedir ? homedir : "", MAX_PATH_LEN);
+	strncat(rcfile, "\\_gnokiirc", MAX_PATH_LEN);
+#else
+	const char *globalrc = "/etc/gnokiirc";
+
+	homedir = getenv("HOME");
+	if (homedir) strncpy(rcfile, homedir, MAX_PATH_LEN);
+	strncat(rcfile, "/.gnokiirc", MAX_PATH_LEN);
+#endif
+
+	/* Try opening .gnokirc from users home directory first */
+	if (gn_cfg_file_read(rcfile)) {
+		/* It failed so try for /etc/gnokiirc */
+		if (gn_cfg_file_read(globalrc)) {
+			/* That failed too so exit */
+			fprintf(stderr, _("Couldn't open %s or %s.\n"), rcfile, globalrc);
+			return -1;
+		}
+	}
+	return 0;
+}
+
+/* DEPRECATED */
+API int gn_cfg_read(char **bindir)
+{
+	int retval;
+
+	retval = gn_cfg_read_default();
+
+	(char *)*bindir = gn_cfg_get(gn_cfg_info, "global", "bindir");
+	if (!*bindir)
+		*bindir = gn_cfg_get(gn_cfg_info, "gnokiid", "bindir");
+	if (!*bindir)
+		*bindir = "/usr/local/sbin";
+
+	return retval;
+}
+
+static int cfg_file_or_memory_read(const char *file, const char **lines)
+{
 	char *val;
 
+	if (file == NULL && lines == NULL) {
+		dprintf("Couldn't open a config file or memory.\n");
+		return -1;
+	}
+
 	/* I know that it doesn't belong here but currently there is now generic
 	 * application init function anywhere.
 	 */
 	setvbuf(stdout, NULL, _IONBF, 0);
 	setvbuf(stderr, NULL, _IONBF, 0);
 
-#ifdef WIN32
-	homedir = getenv("HOMEDRIVE");
-	strncpy(rcfile, homedir ? homedir : "", 200);
-	homedir = getenv("HOMEPATH");
-	strncat(rcfile, homedir ? homedir : "", 200);
-	strncat(rcfile, "\\_gnokiirc", 200);
-#else
-	homedir = getenv("HOME");
-	if (homedir) strncpy(rcfile, homedir, 200);
-	strncat(rcfile, "/.gnokiirc", 200);
-#endif
+	/*
+	 * Try opening a given config file
+	 */
+	if (file != NULL)
+		gn_cfg_info = cfg_file_read(file);
+	else
+		gn_cfg_info = cfg_memory_read(lines);
 
-#ifdef WIN32
-	/* Try opening .gnokirc from users home directory first */
-	if ((gn_cfg_info = cfg_file_read(rcfile)) == NULL) {
-		/* It failed so try for gnokiirc */
-		if ((gn_cfg_info = cfg_file_read("gnokiirc")) == NULL) {
-			/* That failed too so exit */
-			fprintf(stderr, _("Couldn't open %s or gnokiirc. Exiting now...\n"), rcfile);
-			return -1;
-		}
+	if (gn_cfg_info == NULL) {
+		/* this is bad, but the previous was much worse - bozo */
+		dprintf("Couldn't open %s config file,\n", file);
+		return -1;
 	}
-#else
-	/* Try opening .gnokirc from users home directory first */
-	if ((gn_cfg_info = cfg_file_read(rcfile)) == NULL) {
-		/* It failed so try for /etc/gnokiirc */
-		if ((gn_cfg_info = cfg_file_read("/etc/gnokiirc")) == NULL) {
-			/* That failed too so exit */
-			fprintf(stderr, _("Couldn't open %s or /etc/gnokiirc. Exiting now...\n"), rcfile);
-			return -1;
-		}
-	}
-#endif
-
 	strcpy(gn_config_default.model, "");
 	strcpy(gn_config_default.port_device, "");
 	gn_config_default.connection_type = GN_CT_Serial;
@@ -469,9 +603,12 @@ API int gn_cfg_read(char **bindir)
 	strcpy(gn_config_default.connect_script, "");
 	strcpy(gn_config_default.disconnect_script, "");
 	gn_config_default.rfcomm_cn = 1;
+	gn_config_default.sm_retry = 0;
 
-	if (!cfg_psection_load(&gn_config_global, "global", &gn_config_default))
+	if (!cfg_psection_load(&gn_config_global, "global", &gn_config_default)) {
+		fprintf(stderr, _("No global section in %s config file.\n"), file);
 		return -2;
+	}
 
 	/* hack to support [sms] / smsc_timeout parameter */
 	if (gn_config_global.smsc_timeout < 0) {
@@ -481,9 +618,6 @@ API int gn_cfg_read(char **bindir)
 			gn_config_global.smsc_timeout = 10 * atoi(val);
 	}
 
-	(char *)*bindir = gn_cfg_get(gn_cfg_info, "global", "bindir");
-	if (!*bindir) (char *)*bindir = default_bindir;
-
 	if (!cfg_get_log_target(&gn_log_debug_mask, "debug")) return -2;
 	if (!cfg_get_log_target(&gn_log_rlpdebug_mask, "rlpdebug")) return -2;
 	if (!cfg_get_log_target(&gn_log_xdebug_mask, "xdebug")) return -2;
@@ -494,6 +628,16 @@ API int gn_cfg_read(char **bindir)
 	return 0;
 }
 
+API int gn_cfg_file_read(const char *file)
+{
+	return cfg_file_or_memory_read(file, NULL);
+}
+
+API int gn_cfg_memory_read(const char **lines)
+{
+	return cfg_file_or_memory_read(NULL, lines);
+}
+
 API bool gn_cfg_phone_load(const char *iname, struct gn_statemachine *state)
 {
 	char section[256];
