@@ -1,10 +1,10 @@
 /*
 
-  $Id: compat.c,v 1.8 2003/02/16 22:11:38 godehardt Exp $
+  $Id: compat.c,v 1.9 2005/04/20 22:33:36 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2002, BORBELY Zoltan
+  Copyright (C) 2002 BORBELY Zoltan, Markus Plail, Pawel Kot
 
   Various compatibility functions.
 
