@@ -1,10 +1,10 @@
 /*
 
-  $Id: at-emulator.c,v 1.38 2003/12/16 00:56:27 bozo Exp $
+  $Id: at-emulator.c,v 1.42 2005/04/20 20:30:15 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,9 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000  Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2001-2004  Pawel Kot
+  Copyright (C) 2002-2004  BORBELY Zoltan
 
   This file provides a virtual modem or "AT" interface to the GSM phone by
   calling code in gsm-api.c. Inspired by and in places copied from the Linux
@@ -39,6 +41,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <time.h>
 #include <unistd.h>
 #include <ctype.h>
 
@@ -107,6 +110,7 @@ static int	CurrentCmdBuffer;
 static int	CurrentCmdBufferIndex;
 static int	IncomingCallNo;
 static int     MessageFormat;          /* Message Format (text or pdu) */
+static int	CallerIDMode;
 
 	/* Current command parser */
 static void 	(*Parser)(char *); /* Current command parser */
@@ -181,6 +185,8 @@ void	gn_atem_registers_init(void)
 	ModemRegisters[REG_VERBOSE] |= BIT_VERBOSE;
 	ModemRegisters[REG_CTRLZ] = 26;
 	ModemRegisters[REG_ESCAPE] = 27;
+
+	CallerIDMode = 0;
 }
 
 
@@ -221,6 +227,7 @@ void gn_atem_call_passup(gn_call_status CallStatus, gn_call_info *CallInfo, stru
 		gn_atem_modem_result(MR_RING);
 		IncomingCallNo = CallInfo->call_id;
 		ModemRegisters[REG_RINGCNT]++;
+		gn_atem_cid_out(CallInfo);
 		if (ModemRegisters[REG_RINGATA] != 0) gn_atem_answer_phone();
 		break;
 	case GN_CALL_LocalHangup:
@@ -232,6 +239,40 @@ void gn_atem_call_passup(gn_call_status CallStatus, gn_call_info *CallInfo, stru
 	}
 }
 
+/* This gets called to output caller id info of incoming call */
+void gn_atem_cid_out(gn_call_info *CallInfo)
+{
+	struct tm *now;
+	time_t nowh;
+	char buf[14]; /* 7 for "DATE = " + 4 digits + \n + \r + \0 */
+
+	nowh = time(NULL);
+	now = localtime(&nowh);
+
+	switch (CallerIDMode) {
+	case 0:	/* no output */
+		break;
+	case 1: /* formatted CID */
+		snprintf(buf, sizeof(buf), "DATE = %02d%02d\n\r", now->tm_mon + 1, now->tm_mday);
+		gn_atem_string_out(buf);
+
+		snprintf(buf, sizeof(buf), "TIME = %02d%02d\n\r", now->tm_hour, now->tm_min);
+		gn_atem_string_out(buf);
+
+		/* TO DO: handle P and O numbers */
+		gn_atem_string_out("NMBR = ");
+		gn_atem_string_out(1 + CallInfo->number); /* skip leading "+" */
+		gn_atem_string_out("\n\rNAME = ");
+		gn_atem_string_out(CallInfo->name);
+		gn_atem_string_out("\n\r");
+
+		/* FIX ME: do a real emulation of rings after the first one (at a lower level than this) */
+		gn_atem_modem_result(MR_RING);
+
+		break;
+
+	}
+}
 
 /* Handler called when characters received from serial port.
    calls state machine code to process it. */
@@ -304,6 +345,8 @@ void	gn_atem_at_parse(char *cmd_buffer)
 	int regno, val;
 	char str[256];
 
+	if (!cmd_buffer[0]) return;
+
 	if (strncasecmp (cmd_buffer, "AT", 2) != 0) {
 		gn_atem_modem_result(MR_ERROR);
 		return;
@@ -544,6 +587,15 @@ void	gn_atem_at_parse(char *cmd_buffer)
 			}
 			break;
 
+		/* # is the precursor to another set of commands */
+		case '#':
+			buf++;
+			/* Returns true if error occured */
+			if (gn_atem_command_diesis(&buf) == true) {
+				return;
+			}
+			break;
+
 		default:
 			gn_atem_modem_result(MR_ERROR);
 			return;
@@ -558,14 +610,30 @@ static void gn_atem_sms_print(char *line, gn_sms *message, int mode)
 {
 	switch (mode) {
 	case INTERACT_MODE:
-		gsprintf(line, MAX_LINE_LENGTH, _("\n\rDate/time: %d/%d/%d %d:%02d:%02d Sender: %s Msg Center: %s\n\rText: %s\n\r"), message->time.day, message->time.month, message->time.year, message->time.hour, message->time.minute, message->time.second, message->remote.number, message->smsc.number, message->user_data[0].u.text);
+		gsprintf(line, MAX_LINE_LENGTH,
+			_("\n\rDate/time: %d/%d/%d %d:%02d:%02d Sender: %s Msg Center: %s\n\rText: %s\n\r"),
+			message->smsc_time.day, message->smsc_time.month, message->smsc_time.year,
+			message->smsc_time.hour, message->smsc_time.minute, message->smsc_time.second,
+			message->remote.number, message->smsc.number, message->user_data[0].u.text);
 		break;
 	case TEXT_MODE:
 		if ((message->dcs.type == GN_SMS_DCS_GeneralDataCoding) &&
 		    (message->dcs.u.general.alphabet == GN_SMS_DCS_8bit))
-			gsprintf(line, MAX_LINE_LENGTH, _("\"%s\",\"%s\",,\"%02d/%02d/%02d,%02d:%02d:%02d+%02d\"\n\r%s"), (message->status ? _("REC READ") : _("REC UNREAD")), message->remote.number, message->time.year, message->time.month, message->time.day, message->time.hour, message->time.minute, message->time.second, message->time.timezone, _("<Not implemented>"));
+			gsprintf(line, MAX_LINE_LENGTH,
+				_("\"%s\",\"%s\",,\"%02d/%02d/%02d,%02d:%02d:%02d+%02d\"\n\r%s"),
+				(message->status ? _("REC READ") : _("REC UNREAD")),
+				message->remote.number,
+				message->smsc_time.year, message->smsc_time.month, message->smsc_time.day,
+				message->smsc_time.hour, message->smsc_time.minute, message->smsc_time.second,
+				message->time.timezone, _("<Not implemented>"));
 		else
-			gsprintf(line, MAX_LINE_LENGTH, _("\"%s\",\"%s\",,\"%02d/%02d/%02d,%02d:%02d:%02d+%02d\"\n\r%s"), (message->status ? _("REC READ") : _("REC UNREAD")), message->remote.number, message->time.year, message->time.month, message->time.day, message->time.hour, message->time.minute, message->time.second, message->time.timezone, message->user_data[0].u.text);
+			gsprintf(line, MAX_LINE_LENGTH,
+				_("\"%s\",\"%s\",,\"%02d/%02d/%02d,%02d:%02d:%02d+%02d\"\n\r%s"),
+				(message->status ? _("REC READ") : _("REC UNREAD")),
+				message->remote.number,
+				message->smsc_time.year, message->smsc_time.month, message->smsc_time.day,
+				message->smsc_time.hour, message->smsc_time.minute, message->smsc_time.second,
+				message->time.timezone, message->user_data[0].u.text);
 		break;
 	case PDU_MODE:
 		gsprintf(line, MAX_LINE_LENGTH, _("<Not implemented>"));
@@ -991,6 +1059,39 @@ bool	gn_atem_command_plusg(char **buf)
 	return (true);
 }
 
+/* Handle AT# commands */
+bool	gn_atem_command_diesis(char **buf)
+{
+	int	number;
+	char	buffer[MAX_LINE_LENGTH];
+
+	if (strncasecmp(*buf, "CID", 3) == 0) {
+		buf[0] += 3;
+		switch (**buf) {
+		case '?':
+			buf[0]++;
+			gsprintf(buffer, MAX_LINE_LENGTH, "%d\n\r", CallerIDMode);
+			gn_atem_string_out(buffer);
+			return (false);
+		case '=':
+			buf[0]++;
+			if (**buf == '?') {
+				buf[0]++;
+				gn_atem_string_out("0,1\n\r");
+				return (false);
+			} else {
+				number = gn_atem_num_get(buf);
+				if ( number >= 0 && number <= 2 ) {
+					CallerIDMode = number;
+					return (false);
+				}
+			}
+		}
+	}
+	gn_atem_modem_result(MR_ERROR);
+	return (true);
+}
+
 /* Send a result string back.  There is much work to do here, see
    the code in the isdn driver for an idea of where it's heading... */
 void	gn_atem_modem_result(int code)
