@@ -1,10 +1,10 @@
 /*
 
-  $Id: virtmodem.c,v 1.41 2003/10/23 17:45:41 bozo Exp $
+  $Id: virtmodem.c,v 1.43 2005/04/20 20:30:18 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,10 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000  Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2002       Ladis Michl, Manfred Jonsson, Jan Kratochvil
+  Copyright (C) 2001-2004  Pawel Kot
+  Copyright (C) 2002-2003  BORBELY Zoltan
 
   This file provides a virtual modem interface to the GSM phone by calling
   code in gsm-api.c, at-emulator.c and datapump.c. The code here provides
@@ -52,6 +55,11 @@
 #  define	__BSD_VISIBLE	1
 #endif
 
+/* same for NetBSD */
+#ifdef	__NetBSD__
+#  include <sys/types.h>
+#endif
+
 /* This is the correct way to include stdlib with _XOPEN_SOURCE = 500 defined.
  * Needed for clean unlockpt() declaration.
  * msghdr structure in Solaris depends on _XOPEN_SOURCE = 500 too. - bozo
