@@ -1,10 +1,10 @@
 /*
 
-  $Id: device.c,v 1.34 2004/06/19 23:20:34 pkot Exp $
+  $Id: device.c,v 1.36 2005/04/20 22:33:36 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,11 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2001      Chris Kemp
+  Copyrught (C) 2001-2004 Pawel Kot
+  Copyright (C) 2002-2003 BORBELY Zoltan
+  Copyright (C) 2002      Pavel Machek, Marcin Wiacek
 
 */
 
@@ -56,6 +60,7 @@ int device_open(const char *file, int with_odd_parity, int with_async,
 	dprintf("Serial device: opening device %s\n", file);
 
 	switch (state->device.type) {
+	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
 		state->device.fd = serial_opendevice(file, with_odd_parity, with_async, with_hw_handshake, state);
@@ -86,6 +91,7 @@ void device_close(struct gn_statemachine *state)
 	dprintf("Serial device: closing device\n");
 
 	switch (state->device.type) {
+	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
 		serial_close(state->device.fd, state);
@@ -124,6 +130,7 @@ void device_setdtrrts(int dtr, int rts, struct gn_statemachine *state)
 	dprintf("Serial device: setting RTS to %s and DTR to %s\n", rts ? "high" : "low", dtr ? "high" : "low");
 
 	switch (state->device.type) {
+	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
 		serial_setdtrrts(state->device.fd, dtr, rts, state);
@@ -148,6 +155,7 @@ void device_changespeed(int speed, struct gn_statemachine *state)
 	dprintf("Serial device: setting speed to %d\n", speed);
 
 	switch (state->device.type) {
+	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
 		serial_changespeed(state->device.fd, speed, state);
@@ -171,6 +179,7 @@ void device_changespeed(int speed, struct gn_statemachine *state)
 size_t device_read(__ptr_t buf, size_t nbytes, struct gn_statemachine *state)
 {
 	switch (state->device.type) {
+	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
 		return (serial_read(state->device.fd, buf, nbytes, state));
@@ -193,6 +202,7 @@ size_t device_read(__ptr_t buf, size_t nbytes, struct gn_statemachine *state)
 size_t device_write(const __ptr_t buf, size_t n, struct gn_statemachine *state)
 {
 	switch (state->device.type) {
+	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
 		return (serial_write(state->device.fd, buf, n, state));
@@ -215,6 +225,7 @@ size_t device_write(const __ptr_t buf, size_t n, struct gn_statemachine *state)
 int device_select(struct timeval *timeout, struct gn_statemachine *state)
 {
 	switch (state->device.type) {
+	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
 		return serial_select(state->device.fd, timeout, state);
@@ -239,6 +250,7 @@ gn_error device_nreceived(int *n, struct gn_statemachine *state)
 	*n = -1;
 
 	switch (state->device.type) {
+	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
 		return serial_nreceived(state->device.fd, n, state);
@@ -250,6 +262,7 @@ gn_error device_nreceived(int *n, struct gn_statemachine *state)
 gn_error device_flush(struct gn_statemachine *state)
 {
 	switch (state->device.type) {
+	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
 		return serial_flush(state->device.fd, state);
