@@ -1,13 +1,10 @@
 /*
  *
- * $Id: unixirda.c,v 1.28 2004/06/19 23:20:45 pkot Exp $
+ * $Id: unixirda.c,v 1.36 2005/04/20 20:37:41 pkot Exp $
  *
  * G N O K I I
  *
- * A Linux/Unix toolset and driver for Nokia mobile phones.
- *
- * Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
- * Copyright (C) 2000-2001  Marcel Holtmann <marcel@holtmann.org>
+ * A Linux/Unix toolset and driver for the mobile phones.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +20,13 @@
  * License along with this library; if not, write to the Free
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
+ * Copyright (C) 1999-2000  Hugh Blemings & Pavel Jank ml.
+ * Copyright (C) 2000-2001  Marcel Holtmann <marcel@holtmann.org>
+ * Copyright (C) 2001       Chris Kemp
+ * Copyright (C) 2001-2004  Pawel Kot
+ * Copyright (C) 2002-2004  BORBELY Zoltan
+ * Copyright (C) 2004       Phil Ashby
+ *
  */
 
 #include "config.h"
@@ -58,19 +62,30 @@ static char *phone[] = {
 	"Nokia 5100",
 	"Nokia 6100",
 	"Nokia 6210",
+	"Nokia 6230",
 	"Nokia 6250",
 	"Nokia 6310",
 	"Nokia 6310i",
 	"Nokia 6360",
 	"Nokia 6510",
 	"Nokia 6610",
+	"Nokia 6610i",
+	"Nokia 6650",
+	"Nokia 6800",
+	"Nokia 6810",
+	"Nokia 6820",
+	"Nokia 6820b",
 	"Nokia 7110",
 	"Nokia 7190",
+	"Nokia 7210",
+	"Nokia 7250",
+	"Nokia 7250i",
 	"Nokia 7650",
 	"Nokia 8210",
 	"Nokia 8250",
 	"Nokia 8290",
 	"Nokia 8310",
+	"Nokia 8850",
 	"Nokia 9110",
 	"Nokia 9210"
 };
