@@ -1,10 +1,10 @@
 /*
 
-  $Id: unixserial.c,v 1.35 2003/04/28 14:55:26 pkot Exp $
+  $Id: unixserial.c,v 1.37 2005/04/20 20:37:41 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,12 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
-
+  Copyright (C) 1999-2000  Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2001       Chris Kemp, Manfred Jonsson, Jank Kratochvil
+  Copyright (C) 2002       Ladis Michl, Pavel Machek
+  Copyright (C) 2001-2004  Pawel Kot
+  Copyright (C) 2002-2004  BORBELY Zoltan
+  
 */
 
 #include "misc.h"
@@ -453,8 +457,7 @@ size_t serial_write(int fd, const __ptr_t buf, size_t n, struct gn_statemachine
 		got = write(fd, buf + r, bs);
 		if (got == 0) {
 			dprintf("Serial write: oops, zero byte has written!\n");
-		}
-		else if (got < 0) {
+		} else if (got < 0) {
 			if (errno == EINTR) continue;
 			if (errno != EAGAIN) {
 				dprintf("Serial write: write error %d\n", errno);
