@@ -1,10 +1,10 @@
 /*
 
-  $Id: gsm-encoding.c,v 1.59 2004/07/04 14:11:38 pkot Exp $
+  $Id: gsm-encoding.c,v 1.63 2005/04/20 22:33:36 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,11 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2001 Pawe Kot <pkot@linuxnews.pl>
+  Copyright (C) 1999-2000 Pavel Janik ml.
+  Copyright (C) 2001-2005 Pawel Kot
+  Copyright (C) 2002      Markus Plail, Manfred Jonsson
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2003      Martin Goldhahn
 
   Functions for encoding SMS, calendar and other things.
 
@@ -282,6 +286,7 @@ skip:
 	return (out_num - output);
 }
 
+/* null terminates the string */
 void char_ascii_decode(unsigned char* dest, const unsigned char* src, int len)
 {
 	int i, j;
@@ -311,6 +316,7 @@ unsigned int char_ascii_encode(unsigned char* dest, const unsigned char* src, un
 	return i;
 }
 
+/* null terminates the string */
 void char_hex_decode(unsigned char* dest, const unsigned char* src, int len)
 {
 	int i;
@@ -370,6 +376,7 @@ int char_uni_alphabet_decode(wchar_t value, unsigned char *dest, MBSTATE *mbs)
 	}
 }
 
+/* null terminates the string */
 void char_ucs2_decode(unsigned char* dest, const unsigned char* src, int len)
 {
 	int i_len = 0, o_len = 0, length;
@@ -420,6 +427,7 @@ void char_ucs2_encode(unsigned char* dest, const unsigned char* src, int len)
 	return;
 }
 
+/* null terminates the string */
 unsigned int char_unicode_decode(unsigned char* dest, const unsigned char* src, int len)
 {
 	int i, length = 0, pos = 0;
@@ -519,8 +527,9 @@ int char_semi_octet_pack(char *number, unsigned char *output, gn_gsm_number_type
 	   only international and unknown number. */
 
 	*out_num++ = type;
-	if (type == GN_GSM_NUMBER_International) in_num++; /* Skip '+' */
-	if ((type == GN_GSM_NUMBER_Unknown) && (*in_num == '+')) in_num++; /* Optional '+' in Unknown number type */
+
+	if ((type == GN_GSM_NUMBER_International || type == GN_GSM_NUMBER_Unknown) && *in_num == '+')
+		in_num++; /* skip leading '+' */
 
 	/* The next field is the number. It is in semi-octet representation - see
 	   GSM scpecification 03.40 version 6.1.0, section 9.1.2.3, page 31. */
@@ -857,3 +866,84 @@ int utf8_base64_decode(char *dest, int destlen, const char *in, int inlen)
 	free(aux);
 	return retval;
 }
+
+/* We are escaping the following characters (according to rfc 2426):
+ * '\n', '\r', ';', ',', '\'.
+ */
+int add_slashes(char *dest, char *src, int maxlen, int len)
+{
+	int i, j;
+
+	for (i = 0, j = 0; i < len && j < maxlen; i++, j++) {
+		switch (src[i]) {
+		case '\n':
+			dest[j++] = '\\';
+			dest[j] = 'n';
+			break;
+		case '\r':
+			dest[j++] = '\\';
+			dest[j] = 'r';
+			break;
+		case '\\':
+		case ';':
+		case ',':
+			dest[j++] = '\\';
+		default:
+			dest[j] = src[i];
+			break;
+		}
+	}
+	dest[j] = 0;
+	return j;
+}
+
+int strip_slashes(char *dest, char *src, int maxlen, int len)
+{
+	int i, j, slash_state = 0;
+
+	for (i = 0, j = 0; i < len && j < maxlen; i++) {
+		switch (src[i]) {
+		case ';':
+		case ',':
+			if (slash_state) {
+				dest[j++] = src[i];
+				slash_state = 0;
+			}
+			break;
+		case '\\':
+			if (slash_state) {
+				dest[j++] = src[i];
+				slash_state = 0;
+			} else {
+				slash_state = 1;
+			}
+			break;
+		case 'n':
+			if (slash_state) {
+				dest[j++] = '\n';
+				slash_state = 0;
+			} else {
+				dest[j++] = src[i];
+			}
+			break;
+		case 'r':
+			if (slash_state) {
+				dest[j++] = '\r';
+				slash_state = 0;
+			} else {
+				dest[j++] = src[i];
+			}
+			break;
+		default:
+			if (slash_state) {
+				dest[j++] = '\\';
+				slash_state = 0;
+			} else {
+				dest[j++] = src[i];
+			}
+			break;
+		}
+	}
+	dest[j] = 0;
+	return j;
+}
