@@ -1,10 +1,10 @@
 /*
 
-  $Id: gsm-networks.c,v 1.36 2003/12/30 00:24:54 uid66843 Exp $
+  $Id: gsm-networks.c,v 1.41 2005/04/20 22:33:36 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,9 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2002-2003 Pawel Kot, Ladis Michl
+  Copyright (C) 2002      Feico de Boer
 
   This file implements GSM networks searching.
 
@@ -285,7 +287,7 @@ static gn_network networks[] = {
 	{"232 10", "3 AT"},
 	{"234 10", "O2"},
 	{"234 15", "Vodafone"},
-	{"234 20", "3"},
+	{"234 20", "3 UK"},
 	{"234 30", "T-Mobile UK"},
 	{"234 33", "ORANGE"},
 	{"234 50", "JT GSM"},
@@ -802,6 +804,20 @@ API char *gn_network_code_get(char *network_name)
 	return networks[index].code ? networks[index].code : _("undefined");
 }
 
+API char *gn_network_code_find(char *network_name, char *country_name)
+{
+	int index = 0;
+	char country_code[5];
+	
+	snprintf(country_code, 4, "%3s ", gn_country_code_get(country_name));
+	country_code[4] = 0;
+	while (networks[index].name &&
+	       (!strstr(networks[index].code, country_code) ||
+	        strcasecmp(networks[index].name, network_name))) index++;
+
+	return networks[index].code ? networks[index].code : _("undefined");
+}
+
 API char *gn_country_name_get(char *country_code)
 {
 	int index = 0;
@@ -837,3 +853,13 @@ API bool gn_country_get(gn_country *country, int index)
 	*country = countries[index];
 	return true;
 }
+
+API char *gn_network2country(char *network_code)
+{
+	char ccode[4];
+	
+	strncpy((char *)ccode, network_code, 3);
+	ccode[3] = 0;
+
+	return gn_country_name_get(ccode);
+}
