@@ -1,10 +1,10 @@
 /*
 
-  $Id: atbus.c,v 1.42 2004/06/09 13:58:15 bozo Exp $
+  $Id: atbus.c,v 1.44 2005/04/20 20:43:32 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,11 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright 2001 Manfred Jonsson <manfred.jonsson@gmx.de>
+  Copyright (C) 2001-2002 Manfred Jonsson <manfred.jonsson@gmx.de>
+  Copyright (C) 2001-2003 Ladis Michl
+  Copyright (C) 2001-2004 Pawel Kot
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2002 Pavel Machek
 
 */
 
@@ -120,7 +124,7 @@ static gn_error at_send_message(unsigned int message_length, unsigned char messa
 
 char *findcrlfbw(unsigned char *str, int len)
 {
-	while ((*str != '\n') && (*str-1 != '\r') && len--)
+	while (len-- && (*str != '\n') && (*str-1 != '\r'))
 		str--;
 	return len > 0 ? str+1 : NULL;
 }
