@@ -1,10 +1,10 @@
 /*
 
-  $Id: misc.c,v 1.80 2004/06/10 15:18:50 bozo Exp $
+  $Id: misc.c,v 1.92 2005/04/20 22:33:36 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,18 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2000      Jan Derfinak
+  Copyright (C) 2000-2004 Pawel Kot
+  Copyright (C) 2001      Jan Kratochvil, Ladis Michl, Chris Kemp
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2002      Manfred Jonsson, Markus Plail
+
+  Some parts of the gn_lock_device() function are derived from minicom
+  communications package, main.c file written by Miquel van Smoorenburg.
+  These parts are copyrighted under GNU GPL version 2.
+
+  Copyright 1991-1995 Miquel van Smoorenburg.
 
 */
 
@@ -205,7 +216,7 @@ static gn_phone_model models[] = {
 	{"6210",  "NPE-3", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_NETMONITOR | PM_EXTPBK | PM_SMS | PM_FOLDERS },
 	{"6220",  "RH-20", 0 },
 	{"6225",  "RH-27", 0 },
-	{"6230",  "RH-12", 0 },
+	{"6230",  "RH-12_28_-", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_NETMONITOR | PM_EXTPBK | PM_SMS | PM_FOLDERS },
 	{"6250",  "NHM-3", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_NETMONITOR | PM_EXTPBK | PM_SMS | PM_FOLDERS },
 	{"6310",  "NPE-4", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS | PM_KEYBOARD | PM_DTMF },
 	{"6310i", "NPL-1", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS | PM_KEYBOARD | PM_DTMF },
@@ -222,21 +233,23 @@ static gn_phone_model models[] = {
 	{"6590i", "NSM-9", 0 },
 	{"6600",  "NHL-10", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_NETMONITOR | PM_EXTPBK | PM_SMS | PM_FOLDERS },
 	{"6610",  "NHL-4U", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS },
-	{"6650",  "NHM-1", 0 },
+	{"6610i", "RM-37", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS },
+	{"6650",  "NHM-1", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS },
 	{"6800",  "NHL-6", 0 },
 	{"6800",  "NSB-9", 0 },
-	{"6810",  "RM-2", 0 },
+	{"6810",  "RM-2", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS },
 	{"6820",  "NHL-9", 0 },
 	{"6820b", "RH-26", 0 },
 	{"7110",  "NSE-5", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_NETMONITOR | PM_EXTPBK | PM_SMS | PM_FOLDERS },
 	{"7160",  "NSW-5", 0 },
 	{"7190",  "NSB-5", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_NETMONITOR | PM_EXTPBK | PM_SMS | PM_FOLDERS },
-	{"7210",  "NHL-4", 0 },
-	{"7250",  "NHL-4J", 0 },
-	{"7250i", "NHL-4JX", 0 },
-	{"7600",  "NMM-3", 0 },
+	{"7210",  "NHL-4", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS },
+	{"7250",  "NHL-4J", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS },
+	{"7250i", "NHL-4JX", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS },
+	{"7600",  "NMM-3", PM_CALLERGROUP | PM_EXTPBK | PM_SMS | PM_FOLDERS | PM_SPEEDDIAL | PM_CALENDAR | PM_DTMF },
 	{"7650",  "NHL-2NA", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_EXTPBK | PM_SMS | PM_FOLDERS },
 	{"8110",  "0423" , PM_SMS | PM_DTMF | PM_DATA }, /* NHE-6BX */
+	{"8110",  "2501", PM_SMS | PM_DTMF | PM_DATA },
 	{"8110",  "NHE-6", PM_SMS | PM_DTMF | PM_DATA },
 	{"8110",  "NHE-6BX", PM_SMS | PM_DTMF | PM_DATA },
 	{"8110i", "0423",  PM_SMS | PM_DTMF | PM_DATA }, /* Guess for NHE-6 */
@@ -255,7 +268,7 @@ static gn_phone_model models[] = {
 	{"8310",  "NHM-7", PM_CALLERGROUP | PM_CALENDAR | PM_SPEEDDIAL | PM_NETMONITOR | PM_EXTPBK | PM_SMS | PM_FOLDERS },
 	{"8390",  "NSB-8", 0 },
 	{"8810",  "NSE-6", PM_SMS | PM_DTMF | PM_DATA },
-	{"8850",  "NSM-2", 0 }, 	
+	{"8850",  "NSM-2", PM_CALLERGROUP | PM_CALENDAR | PM_NETMONITOR | PM_SMS | PM_DTMF | PM_DATA | PM_KEYBOARD},
 	{"8855",  "NSM-4", 0 },
 	{"8860",  "NSW-6", 0 },
 	{"8890",  "NSB-6", 0 },
@@ -545,3 +558,39 @@ void gnokii_strfreev(char **str_array)
 	}
 	free(str_array);
 }
+
+/*
+ * check if the timestamp in dt has valid date and time
+ */
+API int gn_timestamp_isvalid(gn_timestamp dt)
+{
+#define BETWEEN(a, x, y)	((a >= x) && (a <= y))
+	int daynum;
+
+	/* assume that year is OK */
+	switch (dt.month) {
+	case 2:
+		if (((dt.year % 4) == 0) &&
+		    (((dt.year % 100) != 0) ||
+		     ((dt.year % 1000) == 0)))
+			daynum = 29;
+		else
+			daynum = 28;
+		break;
+	case 1:
+	case 3:
+	case 5:
+	case 7:
+	case 8:
+	case 10:
+	case 12:
+		daynum = 31;
+		break;
+	default:
+		daynum = 30;
+		break;
+	}
+	return (BETWEEN(dt.month, 1, 12) && BETWEEN(dt.day, 1, daynum) &&
+		BETWEEN(dt.hour, 0, 24) && BETWEEN(dt.minute, 0, 59) &&
+		BETWEEN(dt.second, 0, 59));
+}
