@@ -1,10 +1,10 @@
 /*
 
-  $Id: nokia-decoding.c,v 1.19 2004/02/17 20:29:51 uid66843 Exp $
+  $Id: nokia-decoding.c,v 1.22 2005/04/20 22:33:36 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,14 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2002 Markus Plail
+  Copyright (C) 1999-2000 Hugh Blemings, Pavel Janik
+  Copyright (C) 2000      Chris Kemp
+  Copyright (C) 2001      Marian Jancar
+  Copyright (C) 2001-2002 Markus Plail
+  Copyright (C) 2001-2003 Ladis Michl
+  Copyright (C) 2001-2004 Pawel Kot
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2002      Marcin Wiacek
 
   This file provides decoding functions specific to the newer 
   Nokia 7110/6510 series.
@@ -51,6 +58,7 @@ gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 		    blockstart[0] != GN_PHONEBOOK_ENTRY_LogoSwitch &&
 		    blockstart[0] != GN_PHONEBOOK_ENTRY_Group &&
 		    blockstart[0] != GN_PHONEBOOK_ENTRY_Pointer &&
+		    blockstart[0] != GN_PHONEBOOK_ENTRY_RingtoneAdv &&
 		    /* the last one can be the case of the bitmap name -- we
 		     * don't have phonebook entry allocated then, but a bitmap;
 		     * we handle this later on
@@ -188,6 +196,22 @@ gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 				data->bitmap->number = blockstart[5] - 1;
 			dprintf("   Group: %d\n", blockstart[5] - 1);
 			break;
+		case GN_PHONEBOOK_ENTRY_RingtoneAdv:   /* Newer ringtones */
+			switch(blockstart[15]) {
+			case 0x01:
+				memcpy(data->bitmap->ringtone_id, blockstart+6, 6);
+				dprintf("   Gallery ringtone id: %02x %02x %02x %02x %02x %02x\n", blockstart[6], blockstart[7], blockstart[8], blockstart[9], blockstart[10], blockstart[11]);
+				data->bitmap->ringtone = -1;
+				break;
+			case 0x07:
+				data->bitmap->ringtone = blockstart[11];
+				dprintf("   Standard ringtone: %d\n", blockstart[11]);
+				break;
+			default:
+				dprintf("   Unknown ringtone entry!\n");
+				break;
+			}
+			break;
 		default:
 			dprintf("Unknown phonebook block %02x\n", blockstart[0]);
 			break;
