@@ -1,10 +1,10 @@
 /*
 
-  $Id: atgen.c,v 1.101 2004/04/11 14:15:13 pkot Exp $
+  $Id: atgen.c,v 1.112 2005/04/20 20:52:45 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,13 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2001 Manfred Jonsson <manfred.jonsson@gmx.de>
-  Copyright (C) 2002 Pawel Kot <pkot@linuxnews.pl>
+  Copyright (C) 2001      Manfred Jonsson <manfred.jonsson@gmx.de>
+  Copyright (C) 2002      Pavel Machek, Petr Cech
+  Copyright (C) 2002-2004 Pawel Kot
+  Copyright (C) 2002-2003 Ladis Michl
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2003-2004 Igor Popik
+  Copyright (C) 2004      Hugo Hass, Ron Yorston
 
   This file provides functions specific to generic AT command compatible
   phones. See README for more details on supported mobile phones.
@@ -42,6 +47,7 @@
 #include "phones/ateric.h"
 #include "phones/atnok.h"
 #include "phones/atsie.h"
+#include "phones/atsoer.h"
 #include "links/atbus.h"
 
 static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state);
@@ -137,7 +143,7 @@ static at_function_init_type at_function_init[] = {
 	{ GN_OP_GetNetworkInfo,        AT_GetNetworkInfo,        ReplyGetNetworkInfo },
 };
 
-static char *strip_quotes(char *s)
+char *strip_quotes(char *s)
 {
 	char *t;
 
@@ -165,11 +171,15 @@ static char *strip_brackets(char *s)
 
 static void reply_simpletext(char *l1, char *l2, char *c, char *t)
 {
+	int i;
+
 	if ((strncmp(l1, c, 5) == 0) && (t != NULL)) {
 		if (strncmp(l2, c, 7) == 0)  {
-			strcpy(t, strip_quotes(l2+7));
+			for (i = 7; isspace(l2[i]); i++) ;
+			strcpy(t, strip_quotes(l2 + i));
 		} else {
-			strcpy(t, l2);
+			for (i = 0; isspace(l2[i]); i++) ;
+			strcpy(t, l2 + i);
 		}
 	}
 }
@@ -196,7 +206,7 @@ gn_driver driver_at = {
 	NULL
 };
 
-static char *memorynames[] = {
+char *memorynames[] = {
 	"ME", /* Internal memory of the mobile equipment */
 	"SM", /* SIM card memory */
 	"FD", /* Fixed dial numbers */
@@ -239,7 +249,8 @@ static gn_error Functions(gn_operation op, gn_data *data, struct gn_statemachine
 }
 
 /* Functions to encode and decode strings */
-static int at_encode(int charset, char *dst, char *src, int len) {
+int at_encode(int charset, char *dst, char *src, int len)
+{
 	switch (charset) {
 	case AT_CHAR_GSM:
 		len = char_ascii_encode(dst, src, len);
@@ -260,24 +271,24 @@ static int at_encode(int charset, char *dst, char *src, int len) {
 	return len;
 }
 
-static void at_decode(int charset, char *dst, char *src, int len) {
+void at_decode(int charset, char *dst, char *src, int len)
+{
 	switch (charset) {
+	/* char_*_decode() functions null terminate the strings */
 	case AT_CHAR_GSM:
 		char_ascii_decode(dst, src, len);
 		break;
 	case AT_CHAR_HEXGSM:
 		char_hex_decode(dst, src, len);
-		len *= 2;
 		break;
 	case AT_CHAR_UCS2:
 		char_ucs2_decode(dst, src, len);
-		len *= 4;
 		break;
 	default:
 		memcpy(dst, src, len);
+		dst[len] = 0;
 		break;
 	}
-	dst[len] = '\0';
 }
 
 at_recv_function_type at_insert_recv_function(int type, at_recv_function_type func, struct gn_statemachine *state)
@@ -320,6 +331,16 @@ at_send_function_type at_insert_send_function(int type, at_send_function_type fu
 	return f;
 }
 
+at_error_function_type at_insert_manufacturer_error_function(at_error_function_type func, struct gn_statemachine *state)
+{
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	at_error_function_type f;
+
+	f = drvinst->manufacturer_error;
+	drvinst->manufacturer_error = func;
+	return f;
+}
+
 static gn_error SoftReset(gn_data *data, struct gn_statemachine *state)
 {
 	if (sm_message_send(4, GN_OP_Init, "ATZ\r", state)) return GN_ERR_NOTREADY;
@@ -332,6 +353,123 @@ static gn_error SetEcho(gn_data *data, struct gn_statemachine *state)
 	return sm_block_no_retry(GN_OP_Init, data, state);
 }
 
+static gn_error SetExtendedError(gn_data *data, struct gn_statemachine *state)
+{
+	if (sm_message_send(10, GN_OP_Init, "AT+CMEE=1\r", state)) return GN_ERR_NOTREADY;
+	return sm_block_no_retry(GN_OP_Init, data, state);
+}
+
+gn_error at_error_get(unsigned char *buffer, struct gn_statemachine *state)
+{
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	int code;
+
+	switch (buffer[0]) {
+	case GN_AT_OK:
+		return GN_ERR_NONE;
+
+	case GN_AT_ERROR:
+		return GN_ERR_UNKNOWN;
+
+	case GN_AT_CMS:
+		code = 256 * buffer[1] + buffer[2];
+		switch (code) {
+		case 300: return GN_ERR_FAILED;		/* ME failure */
+		case 301: return GN_ERR_FAILED;		/* SMS service of ME reserved */
+		case 302: return GN_ERR_FAILED;		/* operation not allowed */
+		case 303: return GN_ERR_NOTSUPPORTED;	/* operation not supported */
+		case 304: return GN_ERR_WRONGDATAFORMAT;/* invalid PDU mode parameter */
+		case 305: return GN_ERR_WRONGDATAFORMAT;/* invalid text mode parameter */
+
+		case 310: return GN_ERR_SIMPROBLEM;	/* SIM not inserted */
+		case 311: return GN_ERR_CODEREQUIRED;	/* SIM PIN required */
+		case 312: return GN_ERR_CODEREQUIRED;	/* PH-SIM PIN required */
+		case 313: return GN_ERR_SIMPROBLEM;	/* SIM failure */
+		case 314: return GN_ERR_TRYAGAIN;	/* SIM busy */
+		case 315: return GN_ERR_SIMPROBLEM;	/* SIM wrong */
+		case 316: return GN_ERR_CODEREQUIRED;	/* SIM PUK required */
+		case 317: return GN_ERR_CODEREQUIRED;	/* SIM PIN2 required */
+		case 318: return GN_ERR_CODEREQUIRED;	/* SIM PUK2 required */
+
+		case 320: return GN_ERR_FAILED;		/* memory failure */
+		case 321: return GN_ERR_INVALIDLOCATION;/* invalid memory index */
+		case 322: return GN_ERR_MEMORYFULL;	/* memory full */
+		
+		case 330: return GN_ERR_FAILED;		/* SMSC address unknown */
+		case 331: return GN_ERR_NOCARRIER;	/* no network service */
+		case 332: return GN_ERR_TIMEOUT;	/* network timeout */
+
+		case 340: return GN_ERR_FAILED;		/* no +CNMA acknowledgement expected */
+
+		case 500: return GN_ERR_UNKNOWN;	/* unknown error */
+
+		default:
+			if (code >= 512 && drvinst->manufacturer_error)
+				return drvinst->manufacturer_error(GN_AT_CMS, code, state);
+			break;
+		}
+		break;
+
+	case GN_AT_CME:
+		code = 256 * buffer[1] + buffer[2];
+		switch (code) {
+		case   0: return GN_ERR_FAILED;		/* phone failure */
+		case   1: return GN_ERR_NOLINK;		/* no connection to phone */
+		case   2: return GN_ERR_BUSY;		/* phone-adaptor link reserved */
+		case   3: return GN_ERR_FAILED;		/* operation not allowed */
+		case   4: return GN_ERR_NOTSUPPORTED;	/* operation not supported */
+		case   5: return GN_ERR_CODEREQUIRED;	/* PH-SIM PIN required */
+		case   6: return GN_ERR_CODEREQUIRED;	/* PH-FSIM PIN required */
+		case   7: return GN_ERR_CODEREQUIRED;	/* PH-FSIM PUK required */
+
+		case  10: return GN_ERR_SIMPROBLEM;	/* SIM not inserted */
+		case  11: return GN_ERR_CODEREQUIRED;	/* SIM PIN required */
+		case  12: return GN_ERR_CODEREQUIRED;	/* SIM PUK required */
+		case  13: return GN_ERR_SIMPROBLEM;	/* SIM failure */
+		case  14: return GN_ERR_TRYAGAIN;	/* SIM busy */
+		case  15: return GN_ERR_SIMPROBLEM;	/* SIM wrong */
+		case  16: return GN_ERR_INVALIDSECURITYCODE;	/* incorrect password */
+		case  17: return GN_ERR_CODEREQUIRED;	/* SIM PIN2 required */
+		case  18: return GN_ERR_CODEREQUIRED;	/* SIM PUK2 required */
+
+		case  20: return GN_ERR_MEMORYFULL;	/* memory full */
+		case  21: return GN_ERR_INVALIDLOCATION;/* invalid index */
+		case  22: return GN_ERR_EMPTYLOCATION;	/* not found */
+		case  23: return GN_ERR_FAILED;		/* memory failure */
+		case  24: return GN_ERR_ENTRYTOOLONG;	/* text string too long */
+		case  25: return GN_ERR_WRONGDATAFORMAT;/* invalid characters in text string */
+		case  26: return GN_ERR_ENTRYTOOLONG;	/* dial string too long */
+		case  27: return GN_ERR_WRONGDATAFORMAT;/* invalid characters in dial string */
+
+		case  30: return GN_ERR_NOCARRIER;	/* no network service */
+		case  31: return GN_ERR_TIMEOUT;	/* network timeout */
+		case  32: return GN_ERR_FAILED;		/* network not allowed - emergency calls only */
+		
+		case  40: return GN_ERR_CODEREQUIRED;	/* network personalisation PIN required */
+		case  41: return GN_ERR_CODEREQUIRED;	/* network personalisation PUK required */
+		case  42: return GN_ERR_CODEREQUIRED;	/* network subset personalisation PIN required */
+		case  43: return GN_ERR_CODEREQUIRED;	/* network subset personalisation PUK required */
+		case  44: return GN_ERR_CODEREQUIRED;	/* service provider personalisation PIN required */
+		case  45: return GN_ERR_CODEREQUIRED;	/* service provider personalisation PUK required */
+		case  46: return GN_ERR_CODEREQUIRED;	/* corporate personalisation PIN required */
+		case  47: return GN_ERR_CODEREQUIRED;	/* corporate personalisation PUK required */
+
+		case 100: return GN_ERR_UNKNOWN;	/* unknown */
+
+		default:
+			if (code >= 512 && drvinst->manufacturer_error)
+				return drvinst->manufacturer_error(GN_AT_CME, code, state);
+			break;
+		}
+		break;
+
+	default:
+		return GN_ERR_INTERNALERROR;	/* shouldn't happen */
+	}
+
+	return GN_ERR_UNKNOWN;
+}
+
 /* StoreDefaultCharset
  *
  * for a correct communication with the phone for phonebook entries or
@@ -385,19 +523,36 @@ gn_error at_memory_type_set(gn_memory_type mt, struct gn_statemachine *state)
 	return ret;
 }
 
+/* GSM 07.05 section 3.2.3 specified the +CPMS (Preferred Message Store) AT
+ * command as taking 3 memory parameters <mem1>, <mem2> and <mem3>.  Each of
+ * these stores is used for different purposes:
+ *   <mem1> - memory from which messages are read and deleted
+ *   <mem2> - memory to which writing and sending operations are made
+ *   <mem3> - memory to which received SMs are preferred to be stored
+ *
+ * To avoid confusing the user by changing <mem3> we set it to the current
+ * setting.
+ */
 gn_error AT_SetSMSMemoryType(gn_memory_type mt, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	gn_data data;
+	gn_sms_status sms_status;
 	char req[32];
 	gn_error ret = GN_ERR_NONE;
 
 	if (mt != drvinst->smsmemorytype) {
-		sprintf(req, "AT+CPMS=\"%s\"\r", memorynames[mt]);
-		ret = sm_message_send(13, GN_OP_Init, req, state);
+		gn_data_clear(&data);
+		data.sms_status = &sms_status;
+		ret = AT_GetSMSStatus(&data, state);
+		if (ret != GN_ERR_NONE)
+			return ret;
+
+		sprintf(req, "AT+CPMS=\"%s\",\"%s\",\"%s\"\r", memorynames[mt], memorynames[mt],
+			memorynames[data.sms_status->new_message_store]);
+		ret = sm_message_send(23, GN_OP_Init, req, state);
 		if (ret != GN_ERR_NONE)
 			return GN_ERR_NOTREADY;
-		gn_data_clear(&data);
 		ret = sm_block_no_retry(GN_OP_Init, &data, state);
 		if (ret == GN_ERR_NONE)
 			drvinst->smsmemorytype = mt;
@@ -472,7 +627,7 @@ static gn_error AT_SetCharset(gn_data *data, struct gn_statemachine *state)
 			drvinst->charset = AT_CHAR_HEXGSM;
 	} else {
 		drvinst->charset = drvinst->defaultcharset;
-		error = (drvinst->charset == AT_CHAR_UNKNOWN) ? GN_ERR_FAILED : GN_ERR_NONE;
+		error = (drvinst->charset == AT_CHAR_UNKNOWN) ? error : GN_ERR_NONE;
 	}
 	return error;
 }
@@ -599,9 +754,9 @@ static gn_error AT_WritePhonebook(gn_data *data, struct gn_statemachine *state)
 	ret = at_memory_type_set(data->phonebook_entry->memory_type, state);
 	if (ret)
 		return ret;
-	if (data->phonebook_entry->empty || (!(*(data->phonebook_entry->name)) && !(*(data->phonebook_entry->number))))
+	if (data->phonebook_entry->empty) {
 		return AT_DeletePhonebook(data, state);
-	else {
+	} else {
 		ret = state->driver.functions(GN_OP_AT_SetCharset, data, state);
 		if (ret)
 			return ret;
@@ -696,7 +851,13 @@ static gn_error AT_GetSMSStatus(gn_data *data, struct gn_statemachine *state)
 
 	if (!data->sms_status) return GN_ERR_INTERNALERROR;
 
-	ret = sm_message_send(13, GN_OP_GetSMSStatus, "AT+CPMS=\"SM\"\r", state);
+        if (data->memory_status) {
+                ret = AT_SetSMSMemoryType(data->memory_status->memory_type,  state);
+                if (ret != GN_ERR_NONE)
+                        return ret;
+        }
+
+	ret = sm_message_send(9, GN_OP_GetSMSStatus, "AT+CPMS?\r", state);
 	if (ret != GN_ERR_NONE)
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_GetSMSStatus, data, state);
@@ -721,6 +882,7 @@ static gn_error AT_WriteSMS(gn_data *data, struct gn_statemachine *state,
 	unsigned char req[10240], req2[5120];
 	gn_error error;
 	unsigned int length, tmp, offset = 0;
+	at_driver_instance *drvinst = AT_DRVINST(state);
 
 	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
 
@@ -733,10 +895,14 @@ static gn_error AT_WriteSMS(gn_data *data, struct gn_statemachine *state,
 	dprintf("PDU mode set\n");
 
 	/* Prepare the message and count the size */
-	memcpy(req2, data->raw_sms->message_center,
-	       data->raw_sms->message_center[0] + 1);
-	offset += data->raw_sms->message_center[0];
-
+	if(drvinst->no_smsc) {
+		/* not even a length byte included */
+		offset--;
+	} else {
+		memcpy(req2, data->raw_sms->message_center,
+		       data->raw_sms->message_center[0] + 1);
+		offset += data->raw_sms->message_center[0];
+	}
 	/* Validity period in relative format */
 	req2[offset + 1] = 0x01 | 0x10;
 	if (data->raw_sms->reject_duplicates) req2[offset + 1] |= 0x04;
@@ -762,7 +928,11 @@ static gn_error AT_WriteSMS(gn_data *data, struct gn_statemachine *state,
 
 	/* Length in AT mode is the length of the full message minus
 	 * SMSC field length */
-	sprintf(req, "AT+%s=%d\r", cmd, length - data->raw_sms->message_center[0] - 1);
+	if(drvinst->no_smsc) {
+		sprintf(req, "AT+%s=%d\r", cmd, length);
+	} else {
+		sprintf(req, "AT+%s=%d\r", cmd, length - data->raw_sms->message_center[0] - 1);
+	}
 	dprintf("Sending initial sequence\n");
 	if (sm_message_send(strlen(req), GN_OP_AT_Prompt, req, state))
 		return GN_ERR_NOTREADY;
@@ -790,6 +960,14 @@ static gn_error AT_GetSMS(gn_data *data, struct gn_statemachine *state)
 
 	if (err)
 		return err;
+
+	err = state->driver.functions(GN_OP_AT_SetPDUMode, data, state);
+	if (err) {
+		dprintf("PDU mode not supported\n");
+		return err;
+	}
+	dprintf("PDU mode set\n");
+
 	sprintf(req, "AT+CMGR=%d\r", data->raw_sms->number);
 	dprintf("%s", req);
 	if (sm_message_send(strlen(req), GN_OP_GetSMS, req, state))
@@ -886,7 +1064,8 @@ static gn_error AT_GetNetworkInfo(gn_data *data, struct gn_statemachine *state)
 	if (sm_message_send(9, GN_OP_GetNetworkInfo, "AT+CREG?\r", state))
 		return GN_ERR_NOTREADY;
 
-	error = sm_block_no_retry(GN_OP_GetNetworkInfo, data, state);
+	if ((error = sm_block_no_retry(GN_OP_GetNetworkInfo, data, state)) != GN_ERR_NONE)
+		return error;
 
 	if (sm_message_send(9, GN_OP_GetNetworkInfo, "AT+COPS?\r", state))
 		return GN_ERR_NOTREADY;
@@ -899,10 +1078,10 @@ static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int l
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	at_line_buffer buf;
 	char *pos, *endpos;
-	int l;
+	gn_error error;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_INVALIDLOCATION;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return (error == GN_ERR_UNKNOWN) ? GN_ERR_INVALIDLOCATION : error;
 
 	buf.line1 = buffer + 1;
 	buf.length = length;
@@ -926,6 +1105,7 @@ static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int l
 	if (data->phonebook_entry) {
 		data->phonebook_entry->caller_group = 0;
 		data->phonebook_entry->subentries_count = 0;
+		data->phonebook_entry->empty = false;
 
 		/* store number */
 		pos = strchr(buf.line2, '\"');
@@ -954,9 +1134,9 @@ static gn_error ReplyGetSMSCenter(int messagetype, unsigned char *buffer, int le
 {
 	at_line_buffer buf;
 	unsigned char *pos, *aux;
+	gn_error error;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_UNKNOWN; /* FIXME */
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
 	buf.line1 = buffer + 1;
 	buf.length= length;
@@ -997,9 +1177,10 @@ static gn_error ReplyMemoryStatus(int messagetype, unsigned char *buffer, int le
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	at_line_buffer buf;
 	char *pos;
+	gn_error error;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_INVALIDMEMORYTYPE;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return (error == GN_ERR_UNKNOWN) ? GN_ERR_INVALIDMEMORYTYPE : error;
 
 	buf.line1 = buffer + 1;
 	buf.length= length;
@@ -1030,12 +1211,12 @@ static gn_error ReplyMemoryRange(int messagetype, unsigned char *buffer, int len
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	at_line_buffer buf;
 	char *pos, *s, *t;
+	gn_error error;
 
 	drvinst->memoryoffset = 0;
 	drvinst->memorysize = 100;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_UNKNOWN;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
 	buf.line1 = buffer + 1;
 	buf.length= length;
@@ -1066,9 +1247,9 @@ static gn_error ReplyGetBattery(int messagetype, unsigned char *buffer, int leng
 {
 	at_line_buffer buf;
 	char *pos;
+	gn_error error;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_UNKNOWN;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
 	buf.line1 = buffer + 1;
 	buf.length= length;
@@ -1099,10 +1280,10 @@ static gn_error ReplyGetRFLevel(int messagetype, unsigned char *buffer, int leng
 {
 	at_line_buffer buf;
 	char *pos1, *pos2;
+	gn_error error;
+
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_UNKNOWN;
-		
 	buf.line1 = buffer + 1;
 	buf.length= length;
 	
@@ -1124,9 +1305,10 @@ static gn_error ReplyGetRFLevel(int messagetype, unsigned char *buffer, int leng
 static gn_error ReplyIdentify(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_line_buffer buf;
+	gn_error error;
+
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_UNKNOWN;		/* FIXME */
 	buf.line1 = buffer + 1;
 	buf.length = length;
 	splitlines(&buf);
@@ -1151,35 +1333,51 @@ static gn_error ReplyCallDivert(int messagetype, unsigned char *buffer, int leng
 
 static gn_error ReplyGetPrompt(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
-	return (buffer[0] == GN_AT_PROMPT) ? GN_ERR_NONE : GN_ERR_INTERNALERROR;
+	switch (buffer[0]) {
+	case GN_AT_PROMPT: return GN_ERR_NONE;
+	case GN_AT_OK: return GN_ERR_INTERNALERROR;
+	default: return at_error_get(buffer, state);
+	}
 }
 
 static gn_error ReplyGetSMSStatus(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_line_buffer buf;
+	gn_error error;
+	char store[3] = "XX";
+	int i;
 
-	if (buffer[0] != GN_AT_OK) return GN_ERR_FAILED;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
 	buf.line1 = buffer + 1;
 	buf.length = length;
 	splitlines(&buf);
 
-	if (sscanf(buf.line2, "+CPMS: %d", &data->sms_status->number) != 1)
+	if (sscanf(buf.line2, "+CPMS: \"%*c%*c\",%d,%*d,\"%*c%*c\",%*d,%*d,\"%c%c\",%*d,%*d",
+		   &data->sms_status->number, &store[0], &store[1]) != 3)
 		return GN_ERR_FAILED;
 
 	data->sms_status->unread = 0;
 	data->sms_status->changed = 0;
 	data->sms_status->folders_count = 0;
 
+	data->sms_status->new_message_store = GN_MT_ME;
+	for (i = 0; i < sizeof(memorynames)/sizeof(char *); i++) {
+		if (strcmp(store, memorynames[i]) == 0) {
+			data->sms_status->new_message_store = i;
+			break;
+		}
+	}
+
 	return GN_ERR_NONE;
 }
 
 static gn_error ReplySendSMS(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_line_buffer buf;
+	gn_error error;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_FAILED;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
 	buf.line1 = buffer + 1;
 	buf.length = length;
@@ -1201,10 +1399,11 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 	gn_error ret = GN_ERR_NONE;
 	unsigned int sms_len, l, offset = 0;
 	char *tmp;
-	
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_INTERNALERROR;
+	gn_error error;
+ 	at_driver_instance *drvinst = AT_DRVINST(state);
 
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
+	
 	buf.line1 = buffer + 1;
 	buf.length = length;
 
@@ -1225,14 +1424,18 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 	}
 	dprintf("%s\n", buf.line3);
 	hex2bin(tmp, buf.line3, sms_len);
-	l = tmp[offset] + 1;
-	if (l > sms_len || l > GN_SMS_SMSC_NUMBER_MAX_LENGTH) {
-		dprintf("Invalid message center length (%d)\n", l);
-		ret = GN_ERR_INTERNALERROR;
-		goto out;
+
+	if(!drvinst->no_smsc) {
+		l = tmp[offset] + 1;
+		if (l > sms_len || l > GN_SMS_SMSC_NUMBER_MAX_LENGTH) {
+			dprintf("Invalid message center length (%d)\n", l);
+			ret = GN_ERR_INTERNALERROR;
+			goto out;
+		}
+		memcpy(data->raw_sms->message_center, tmp, l);
+		offset += l;
 	}
-	memcpy(data->raw_sms->message_center, tmp, l);
-	offset += l;
+
 	data->raw_sms->type                = (tmp[offset] & 0x03) << 1;
 	data->raw_sms->udh_indicator       = tmp[offset];
 	data->raw_sms->more_messages       = tmp[offset];
@@ -1276,9 +1479,9 @@ static gn_error ReplyGetCharset(int messagetype, unsigned char *buffer, int leng
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	at_line_buffer buf;
 	int i;
+	gn_error error;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_UNKNOWN;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
 	buf.line1 = buffer + 1;
 	buf.length= length;
@@ -1310,10 +1513,10 @@ static gn_error ReplyGetSecurityCodeStatus(int messagetype, unsigned char *buffe
 {
 	at_line_buffer buf;
 	char *pos;
+	gn_error error;
+
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_UNKNOWN;
-		
 	buf.line1 = buffer + 1;
 	buf.length= length;
 	splitlines(&buf);
@@ -1356,12 +1559,13 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 	at_line_buffer buf;
 	char *pos;
 	char **strings;
+	gn_error error;
+	int i;
 
 	if (!data->network_info)
 		return GN_ERR_INTERNALERROR;
 
-	if (buffer[0] != GN_AT_OK)
-		return GN_ERR_UNKNOWN;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
 	buf.line1 = buffer + 1;
 	buf.length= length;
@@ -1372,43 +1576,45 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 		char tmp[3] = {0, 0, 0};
 
 		strings = gnokii_strsplit(buf.line2, ",", 4);
+		i = strings[3] ? 2 : 1;
 
-		if (strings[2] && strlen(strings[2]) >= 6) {
-			pos = strings[2];
-			pos++;
-
-			tmp[0] = pos[0];
-			tmp[1] = pos[1];
-
-			data->network_info->LAC[0] = strtol(tmp, NULL, 16);
-
-			tmp[0] = pos[2];
-			tmp[1] = pos[3];
-
-			data->network_info->LAC[1] = strtol(tmp, NULL, 16);
+		if (!strings[i] || strlen(strings[i]) < 6 || !strings[i + 1] || strlen(strings[i + 1]) < 6) {
+			gnokii_strfreev(strings);
+			return GN_ERR_FAILED;
 		}
 
-		if (strings[3] && strlen(strings[3]) >= 6) {
+		pos = strings[i];
+		pos++;
 
-			pos = strings[3];
-			pos++;
+		tmp[0] = pos[0];
+		tmp[1] = pos[1];
 
-			tmp[0] = pos[0];
-			tmp[1] = pos[1];
+		data->network_info->LAC[0] = strtol(tmp, NULL, 16);
 
-			data->network_info->cell_id[0] = strtol(tmp, NULL, 16);
+		tmp[0] = pos[2];
+		tmp[1] = pos[3];
 
-			tmp[0] = pos[2];
-			tmp[1] = pos[3];
+		data->network_info->LAC[1] = strtol(tmp, NULL, 16);
 
-			data->network_info->cell_id[1] = strtol(tmp, NULL, 16);
-		}
+
+		pos = strings[i + 1];
+		pos++;
+
+		tmp[0] = pos[0];
+		tmp[1] = pos[1];
+
+		data->network_info->cell_id[0] = strtol(tmp, NULL, 16);
+
+		tmp[0] = pos[2];
+		tmp[1] = pos[3];
+
+		data->network_info->cell_id[1] = strtol(tmp, NULL, 16);
 
 		gnokii_strfreev(strings);
 
 	} else if (!strncmp(buf.line1, "AT+COPS?", 8)) {
 		char tmp[128];
-		int format, l;
+		int format;
 
 		memset(tmp, 0, sizeof(tmp));
 		strings = gnokii_strsplit(buf.line2, ",", 3);
@@ -1422,7 +1628,15 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 			snprintf(data->network_info->network_code, sizeof(data->network_info->network_code), gn_network_code_get(tmp));
 			break;
 		case 2: /* network operator code given */
-			if (strlen(strings[2]) >= 6) { 
+			if (strlen(strings[2]) == 5) { 
+				data->network_info->network_code[0] = strings[2][0];
+				data->network_info->network_code[1] = strings[2][1];
+				data->network_info->network_code[2] = strings[2][2];
+				data->network_info->network_code[3] = ' ';
+				data->network_info->network_code[4] = strings[2][3];
+				data->network_info->network_code[5] = strings[2][4];
+				data->network_info->network_code[6] = 0;
+			} else if (strlen(strings[2]) >= 6) { 
 				data->network_info->network_code[0] = strings[2][1];
 				data->network_info->network_code[1] = strings[2][2];
 				data->network_info->network_code[2] = strings[2][3];
@@ -1448,7 +1662,7 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
  * for reference */
 static gn_error Reply(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
-	return (buffer[0] != GN_AT_OK) ? GN_ERR_UNKNOWN : GN_ERR_NONE;
+	return at_error_get(buffer, state);
 }
 
 static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
@@ -1470,12 +1684,14 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 
 	state->driver.incoming_functions = drvinst->incoming_functions;
 	AT_DRVINST(state) = drvinst;
+	drvinst->manufacturer_error = NULL;
 	drvinst->memorytype = GN_MT_XX;
 	drvinst->memoryoffset = 0;
 	drvinst->memorysize = 100;
 	drvinst->smsmemorytype = GN_MT_XX;
 	drvinst->defaultcharset = AT_CHAR_UNKNOWN;
 	drvinst->charset = AT_CHAR_UNKNOWN;
+	drvinst->no_smsc = 0;
 
 	drvinst->if_pos = 0;
 	for (i = 0; i < GN_OP_AT_Max; i++) {
@@ -1509,6 +1725,7 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 
 	SoftReset(&data, state);
 	SetEcho(&data, state);
+	SetExtendedError(&data, state);
 
 	/*
 	 * detect manufacturer and model for further initialization
@@ -1531,6 +1748,8 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 		at_nokia_init(model, setupdata->model, state);
 	else if (!strncasecmp(manufacturer, "siemens", 7))
 		at_siemens_init(model, setupdata->model, state);
+	else if (!strncasecmp(manufacturer, "sony ericsson", 14))
+		at_sonyericsson_init(model, setupdata->model, state);
 	
 	StoreDefaultCharset(state);
 
