@@ -1,10 +1,10 @@
 /*
 
-  $Id: nk3110.c,v 1.43 2003/06/14 15:00:56 osma Exp $
+  $Id: nk3110.c,v 1.45 2005/04/20 20:52:45 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -23,7 +23,9 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
   Copyright (C) 2000 Hugh Blemings & Pavel Jank ml.
-  Copyright (C) 2001 Pawe Kot <pkot@linuxnews.pl>
+  Copyright (C) 2001 Pawe Kot
+  Copyright (C) 2003-2003 BORBELY Zoltan
+  Copyright (C) 2003 Osma Suominen
 
   This file provides functions specific to the 3110 series.
   See README for more details on supported mobile phones.
@@ -49,7 +51,7 @@
 #include "gnokii-internal.h"
 
 
-#define DRVINSTANCE(s) ((nk3110_driver_instance *)((s)->driver.driver_instance))
+#define DRVINSTANCE(s) (*((nk3110_driver_instance **)(&(s)->driver.driver_instance)))
 #define FREE(p) do { free(p); (p) = NULL; } while (0)
 
 /* Prototypes */
