@@ -1,10 +1,10 @@
 /*
 
-  $Id: compat.h,v 1.39 2004/06/27 20:36:16 pkot Exp $
+  $Id: compat.h,v 1.41 2005/04/24 19:13:13 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,10 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2002-2004, BORBELY Zoltan
-  Copyright (C) 2002-2004, Pawel Kot
+  Copyright (C) 1999-2000 Hugh Blemings, Pavel Janik
+  Copyright (C) 2002-2004 BORBELY Zoltan, Pawel Kot
+  Copyright (C) 2002      Feico de Boer, Markus Plail
+  Copyright (C) 2003      Marcus Godehardt, Ladis Michl
 
   Header file for various platform compatibility.
 
@@ -214,7 +216,7 @@ int vasprintf(char **ptr, const char *format, va_list ap);
 #endif
 
 /* Get rid of long defines. Use #if __unices__ */
-#define __unices__ defined(__svr4__) || defined(__FreeBSD__) || defined(__bsdi__) || defined(__MACH__) || defined(__OpenBSD__)
+#define __unices__ defined(__svr4__) || defined(__FreeBSD__) || defined(__bsdi__) || defined(__MACH__) || defined(__OpenBSD__) || defined(__NetBSD__)
 
 /* This one is for NLS. */
 #ifdef ENABLE_NLS
