@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii.h,v 1.72 2004/06/17 19:05:43 pkot Exp $
+  $Id: gnokii.h,v 1.78 2005/04/24 20:14:21 pkot Exp $
 
   G N O K I I
 
@@ -22,8 +22,12 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
-  Copyright (C) 2001, 2002, 2003 Pawe Kot
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2001-2005 Pawel Kot
+  Copyright (C) 2001      Chris Kemp
+  copyright (C) 2002      Markus Plail
+  Copyright (C) 2002-2003 BORBELY Zoltan, Ladis Michl
+  Copyright (C) 2004      Martin Goldhahn
   
   Main header file for gnokii. Include just this file in your app.
 */
@@ -57,10 +61,10 @@ typedef unsigned char uint8_t;
 #  endif
 #endif
 
-#define LIBGNOKII_VERSION_STRING "2.1.0"
+#define LIBGNOKII_VERSION_STRING "2.0.2"
 #define LIBGNOKII_VERSION_MAJOR 2
-#define LIBGNOKII_VERSION_MINOR 1
-#define LIBGNOKII_VERSION_RELEASE 0
+#define LIBGNOKII_VERSION_MINOR 0
+#define LIBGNOKII_VERSION_RELEASE 2
 #define LIBGNOKII_MAKE_VERSION( a,b,c ) (((a) << 16) | ((b) << 8) | (c))
 
 #define LIBGNOKII_VERSION \
@@ -86,8 +90,6 @@ extern API struct gn_cfg_header *gn_cfg_info;
 
 /* Files */
 API int gn_file_text_save(char *filename, char *text, int mode);
-/* reads internal gnokii raw phonebook format */
-API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char *buffer);
 
 /* Misc */
 API gn_memory_type gn_str2memory_type(const char *s);
@@ -97,11 +99,13 @@ extern API gn_phone *gn_gsm_info;
 extern API gn_error (*gn_gsm_f)(gn_operation op, gn_data *data,
 			 struct gn_statemachine *state);
 API gn_error gn_gsm_initialise(struct gn_statemachine *sm);
+API int gn_timestamp_isvalid(gn_timestamp dt);
 
 /* Config file */
 API char *gn_cfg_get(struct gn_cfg_header *cfg, const char *section, const char *key);
 API int gn_cfg_read(char **bindir); /* DEPRECATED */
 API int gn_cfg_file_read(const char *filename);
+API int gn_cfg_memory_read(const char **lines);
 API int gn_cfg_read_default();
 API int gn_cfg_phone_load(const char *iname, struct gn_statemachine *state);
 
@@ -112,6 +116,10 @@ API int gn_vcard2phonebook(FILE *f, gn_phonebook_entry *entry);
 API int gn_phonebook2ldif(FILE *f, gn_phonebook_entry *entry);
 API int gn_ldif2phonebook(FILE *f, gn_phonebook_entry *entry);
 
+/* reads internal gnokii raw phonebook format */
+API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char *buffer);
+API gn_error gn_file_phonebook_raw_write(FILE *f, gn_phonebook_entry *entry, char *memory_type_string);
+
 /* DEPRECATED */
 API int gn_vcal_file_event_read(char *filename, gn_calnote *cnote, int number);
 API int gn_vcal_file_todo_read(char *filename, gn_todo *ctodo, int number);
@@ -134,6 +142,7 @@ API void gn_log_debug(const char *fmt, ...);
 API void gn_log_rlpdebug(const char *fmt, ...);
 API void gn_log_xdebug(const char *fmt, ...);
 API void gn_elog_write(const char *fmt, ...);
+typedef API void (*gn_log_func_t)(const char *fmt, ...);
 
 API int gn_line_get(FILE *file, char *line, int count);
 
