@@ -1,10 +1,10 @@
 /*
 
-  $Id: atgen.h,v 1.16 2004/02/18 23:15:55 uid66843 Exp $
+  $Id: atgen.h,v 1.21 2005/04/24 20:05:19 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,11 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright 2001 Manfred Jonsson <manfred.jonsson@gmx.de>
+  Copyright (C) 2000 Hugh Blemings, Pavel Janik
+  Copyright (C) 2001 Manfred Jonsson <manfred.jonsson@gmx.de>
+  Copyright (C) 2002 Pawel Kot
+  Copyright (C) 2003 Ladis Michl
+  Copyright (C) 2004 Ron Yorston, Hugo Hass
 
   This file provides functions specific to generic AT command compatible
   phones. See README for more details on supported mobile phones.
@@ -54,14 +58,18 @@ typedef enum {
 
 typedef gn_error (*at_recv_function_type)(int type, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 typedef gn_error (*at_send_function_type)(gn_data *data, struct gn_statemachine *state);
+typedef gn_error (*at_error_function_type)(int type, int code, struct gn_statemachine *state);
 
 at_recv_function_type at_insert_recv_function(int type, at_recv_function_type func, struct gn_statemachine *state);
 at_send_function_type at_insert_send_function(int type, at_send_function_type func, struct gn_statemachine *state);
+at_error_function_type at_insert_manufacturer_error_function(at_error_function_type func, struct gn_statemachine *state);
 
 typedef struct {
 	at_send_function_type functions[GN_OP_AT_Max];
 	gn_incoming_function_type incoming_functions[GN_OP_AT_Max];
+	at_error_function_type manufacturer_error;
 	int if_pos;
+	int no_smsc;
 
 	gn_memory_type memorytype;
 	int memoryoffset;
@@ -72,7 +80,7 @@ typedef struct {
 	at_charset charset;
 } at_driver_instance;
 
-#define AT_DRVINST(s) ((at_driver_instance *)((s)->driver.driver_instance))
+#define AT_DRVINST(s) (*((at_driver_instance **)(&(s)->driver.driver_instance)))
 
 typedef struct {
 	char *line1;
@@ -83,10 +91,17 @@ typedef struct {
 } at_line_buffer;
 
 gn_error at_memory_type_set(gn_memory_type mt, struct gn_statemachine *state);
+gn_error at_error_get(unsigned char *buffer, struct gn_statemachine *state);
 
+/* There are shared between various AT drivers */
 void splitlines(at_line_buffer *buf);
-
 char *skipcrlf(unsigned char *str);
 char *findcrlf(unsigned char *str, int test, int maxlength);
+char *strip_quotes(char *s);
+
+void at_decode(int charset, char *dst, char *src, int len);
+int at_encode(int charset, char *dst, char *src, int len);
+
+extern char *memorynames[];
 
 #endif
