@@ -1,10 +1,10 @@
 /*
 
-  $Id: xgnokii.c,v 1.73 2004/06/01 21:45:50 pkot Exp $
+  $Id: xgnokii.c,v 1.78 2005/04/24 20:57:37 pkot Exp $
   
   X G N O K I I
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix GUI for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,12 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  & Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 1999      Pavel Jank ml., Hugh Blemings
+  Copyright (C) 1999-2002 Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 2001-2004 Pawel Kot
+  Copyright (C) 2001      Chris Kemp, Marian Jancar
+  Copyright (C) 2002      Markus Plail
+  Copyright (C) 2003-2004 BORBELY Zoltan
 
 */
 
@@ -42,6 +46,10 @@
 #include <time.h>		/* for time   */
 #include <pthread.h>
 
+#ifdef HAVE_SYS_STAT_H
+#  include <sys/stat.h>
+#endif
+
 #ifndef WIN32
 #  include <unistd.h>		/* for usleep */
 #  include <signal.h>
@@ -201,7 +209,7 @@ static gn_sms_message_center tempMessageSettings;
 
 static inline void Help1(GtkWidget * w, gpointer data)
 {
-	gchar *indx = g_strdup_printf("/help/%s/index.html", xgnokiiConfig.locale);
+	gchar *indx = g_strdup_printf("/help/%s/index.html", xgnokiiConfig.help_locale);
 	Help(w, indx);
 	g_free(indx);
 }
@@ -2300,16 +2308,30 @@ static gint RemoveSplash(GtkWidget * Win)
 	return FALSE;
 }
 
-static void ReadConfig(void)
+static bool LocaleHelpExists(gchar *help_locale)
 {
 #ifdef WIN32
-	xgnokiiConfig.locale = "";
+	bool retval = true;
 #else
-	if ((xgnokiiConfig.locale = getenv("LC_ALL")) == NULL)
-		if ((xgnokiiConfig.locale = getenv("LC_MESSAGES")) == NULL)
-			if ((xgnokiiConfig.locale = getenv("LANG")) == NULL)
-				xgnokiiConfig.locale = "POSIX";
+	bool retval;
+	gchar *file;
+	struct stat buf;
+
+	if (!help_locale)
+		return false;
+
+	file = g_strdup_printf("/%s/help/%s", xgnokiiConfig.xgnokiidir, help_locale);
+	if (stat(file, &buf) == 0)
+		retval = true;
+	else
+		retval = false;
+	g_free(file);
 #endif
+	return retval;
+}
+
+static void ReadConfig(void)
+{
 	if (gn_cfg_read_default() < 0) {
 		exit(-1);
 	}
@@ -2320,6 +2342,8 @@ static void ReadConfig(void)
 	asprintf(&xgnokiiConfig.initlength, "%d", statemachine.config.init_length);
 	xgnokiiConfig.connection = statemachine.config.connection_type;
 
+	max_phonebook_number_length = max_phonebook_sim_number_length =
+	    GN_PHONEBOOK_NUMBER_MAX_LENGTH;
 	GUI_ReadXConfig();
 	max_phonebook_name_length = atoi(xgnokiiConfig.maxPhoneLen);
 	max_phonebook_sim_name_length = atoi(xgnokiiConfig.maxSIMLen);
@@ -2327,13 +2351,35 @@ static void ReadConfig(void)
 #ifndef WIN32
 	xgnokiiConfig.xgnokiidir = DefaultXGnokiiDir;
 #endif
-	max_phonebook_number_length = max_phonebook_sim_number_length =
-	    GN_PHONEBOOK_NUMBER_MAX_LENGTH;
 
 	xgnokiiConfig.callerGroups[0] = xgnokiiConfig.callerGroups[1] =
 	    xgnokiiConfig.callerGroups[2] = xgnokiiConfig.callerGroups[3] =
 	    xgnokiiConfig.callerGroups[4] = xgnokiiConfig.callerGroups[5] = NULL;
 	xgnokiiConfig.smsSets = 0;
+#ifdef WIN32
+	xgnokiiConfig.locale = "";
+	xgnokiiConfig.help_locale = "";
+#else
+	if ((xgnokiiConfig.locale = getenv("LC_ALL")) == NULL)
+		if ((xgnokiiConfig.locale = getenv("LC_MESSAGES")) == NULL)
+			if ((xgnokiiConfig.locale = getenv("LANG")) == NULL)
+				xgnokiiConfig.locale = "POSIX";
+	xgnokiiConfig.help_locale = g_strdup(xgnokiiConfig.locale);
+#endif
+	/* we have set en_US.UTF8 locale that is not supported by xgnokii */
+	if (!LocaleHelpExists(xgnokiiConfig.help_locale)) {
+		char *pos = strchr(xgnokiiConfig.locale, '.');
+		g_free(xgnokiiConfig.help_locale);
+		if (pos)
+			xgnokiiConfig.help_locale = g_strndup(xgnokiiConfig.locale,
+							      pos - xgnokiiConfig.locale);
+		else
+			xgnokiiConfig.help_locale = NULL;
+		if (!LocaleHelpExists(xgnokiiConfig.help_locale)) {
+			g_free(xgnokiiConfig.help_locale);
+			xgnokiiConfig.help_locale = g_strdup("en_US");
+		}
+	}
 }
 
 
