@@ -1,10 +1,10 @@
 /*
 
-  $Id: xgnokii.h,v 1.21 2004/01/06 15:46:41 uid66843 Exp $
+  $Id: xgnokii.h,v 1.23 2005/04/24 20:57:37 pkot Exp $
 
   X G N O K I I
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix GUI for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,11 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  & Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 1999      Pavel Jank ml., Hugh Blemings
+  Copyright (C) 1999-2002 Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 2001      Chris Kemp
+  Copyright (C) 2002      Markus Plail
+  Copyright (C) 2002-2004 Pawel Kot
 
 */
 
@@ -60,7 +63,8 @@ typedef struct {
 	gchar *mailbox;		/* Mailbox, where we can save SMS's */
 	gchar *maxSIMLen;	/* Max length of names on SIM card */
 	gchar *maxPhoneLen;	/* Max length of names in phone */
-	gchar *locale;
+	gchar *locale;		/* Locale for the app translations */
+	gchar *help_locale;	/* Locale for HTML help */
 	gn_sms_message_center smsSetting[MAX_SMS_CENTER];
 	UserInf user;
 	gchar *callerGroups[6];
