@@ -1,10 +1,10 @@
 /*
 
-  $Id: xgnokii_calendar.c,v 1.24 2004/02/20 11:02:26 uid66849 Exp $
+  $Id: xgnokii_calendar.c,v 1.26 2005/04/24 20:57:37 pkot Exp $
 
   X G N O K I I
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix GUI for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,10 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  & Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 1999      Pavel Jank ml., Hugh Blemings
+  Copyright (C) 1999-2002 Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 2001-2002 Pawel Kot
+  Copyright (C) 2002      BORBELY Zoltan, Markus Plail
 
 */
 
@@ -113,7 +115,7 @@ static CalTimeDialog calTimeDialog = { NULL, NULL };
 
 static inline void Help1(GtkWidget * w, gpointer data)
 {
-	gchar *indx = g_strdup_printf("/help/%s/windows/calendar/index.html", xgnokiiConfig.locale);
+	gchar *indx = g_strdup_printf("/help/%s/windows/calendar/index.html", xgnokiiConfig.help_locale);
 	Help(w, indx);
 	g_free(indx);
 }
