@@ -1,10 +1,10 @@
 /*
 
-  $Id: xgnokii_lowlevel.h,v 1.29 2003/10/25 13:17:18 bozo Exp $
+  $Id: xgnokii_lowlevel.h,v 1.30 2005/04/24 20:57:37 pkot Exp $
   
   X G N O K I I
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix GUI for the mobile phones.
 
   This file is part of gnokii.
 
@@ -24,6 +24,9 @@
 
   Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
   & Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 2002      Markus Plail
+  Copyright (C) 2002-2003 Pawel Kot, BORBELY Zoltan
+  Copyright (C) 2003      Tomi Ollila
 
 */
 
