@@ -1,10 +1,10 @@
 /*
 
-  $Id: xgnokii_speed.c,v 1.14 2004/06/30 22:24:02 pkot Exp $
+  $Id: xgnokii_speed.c,v 1.17 2005/04/24 20:57:37 pkot Exp $
 
   X G N O K I I
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix GUI for the mobile phones.
 
   This file is part of gnokii.
 
@@ -24,6 +24,8 @@
 
   Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
   & Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 2002-2003 Pawel Kot
+  Copyright (C) 2002      BORBELY Zoltan, Markus Plail
 
 */
 
@@ -58,7 +60,7 @@ static bool speedDialInitialized;
 static inline void Help1(GtkWidget * w, gpointer data)
 {
 	gchar *indx =
-	    g_strdup_printf("/help/%s/windows/speeddial/index.html", xgnokiiConfig.locale);
+	    g_strdup_printf("/help/%s/windows/speeddial/index.html", xgnokiiConfig.help_locale);
 	Help(w, indx);
 	g_free(indx);
 }
@@ -449,7 +451,7 @@ static void OkExportDialog(GtkWidget * w, GtkFileSelection * fs)
 {
 	static YesNoDialog dialog = { NULL, NULL };
 	FILE *f;
-	gchar err[80];
+	gchar err[255];
 
 
 	exportDialogData.fileName = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
@@ -461,7 +463,7 @@ static void OkExportDialog(GtkWidget * w, GtkFileSelection * fs)
 			CreateYesNoDialog(&dialog, YesExportDialog, CancelDialog,
 					  GUI_SpeedDialWindow);
 			gtk_window_set_title(GTK_WINDOW(dialog.dialog), _("Overwrite file?"));
-			g_snprintf(err, 80, _("File %s already exist.\nOverwrite?"),
+			g_snprintf(err, 255, _("File %s already exist.\nOverwrite?"),
 				   exportDialogData.fileName);
 			gtk_label_set_text(GTK_LABEL(dialog.text), err);
 		}
